@@ -598,7 +598,24 @@ class SkipBrokenTests(DepsolveTests):
         self.tsInfo.addUpdate(u1, oldpo=i2)
         self.assertEquals('empty', *self.resolveCode(skip=True))
         self.assertResult([i1,i2])
-        
+ 
+    def testDowngrade1(self):
+        '''
+        bar require foolib=2.0 provided by foo-1.2
+        foo-1.2 is downgraded to foo-1.1 there only contains foolib=1.0
+        so bar requirement is broken and the downgrade should be removed from
+        transaction
+        '''
+        i1 = self.instPackage('foo', '1.2')
+        i1.addProvides('foolib', 'EQ', ('0', '2', '0'))
+        i2 = self.instPackage('bar', '1.0')
+        i2.addRequires('foolib', 'EQ', ('0', '2', '0'))
+        d1 = self.repoPackage('foo', '1.1')
+        d1.addProvides('foolib', 'EQ', ('0', '1', '0'))
+        self.tsInfo.addDowngrade(d1, oldpo=i1)
+        self.assertEquals('empty', *self.resolveCode(skip=True))
+        self.assertResult([i1, i2])
+       
 
     def testMissingfileReqIptabes(self):    
         '''
@@ -621,11 +638,44 @@ class SkipBrokenTests(DepsolveTests):
         self.tsInfo.addUpdate(u1, oldpo=i1)
         self.assertEquals('empty', *self.resolveCode(skip=True))
         self.assertResult([i1,i2])
+
+    def testTransactionOutput(self):
+        '''  
+        Test that skip-broken transaction dump output dont show the 
+        dependon: xxx once.
+        '''
+        i1 = self.repoPackage('bar1', '1')
+        i1.addRequires('foo1', 'EQ', ('0', '1', '0'))
+        i1.addRequires('foo2', 'EQ', ('0', '1', '0'))
+        i1.addRequires('foo3', 'EQ', ('0', '1', '0'))
+        i1.addRequires('foo4', 'EQ', ('0', '1', '0'))
+        i1.addRequires('foo5', 'EQ', ('0', '1', '0'))
+        i1.addRequires('foo6', 'EQ', ('0', '1', '0'))
+        i2 = self.repoPackage('fooA', '1')
+        i2.addProvides('foo1', 'EQ', ('0', '1', '0'))
+        i3 = self.repoPackage('fooB', '1')
+        i3.addProvides('foo2', 'EQ', ('0', '1', '0'))
+        i4 = self.repoPackage('fooC', '1')
+        i4.addProvides('foo3', 'EQ', ('0', '1', '0'))
+        i5 = self.repoPackage('fooD', '1')
+        i5.addProvides('foo4', 'EQ', ('0', '1', '0'))
+        i6 = self.repoPackage('fooE', '1')
+        i6.addProvides('foo5', 'EQ', ('0', '1', '0'))
+        i7 = self.instPackage('fooF', '1')
+        i7.addProvides('foo6', 'EQ', ('0', '1', '0'))
+        u7 = self.instPackage('fooF', '2')
+        u7.addProvides('foo6', 'EQ', ('0', '2', '0'))
+        self.tsInfo.addInstall(i1)
+        self.tsInfo.addUpdate(u7, oldpo=i7)
+        self.assertEquals('ok', *self.resolveCode(skip=True))
+        # uncomment this line and the test will fail and you can see the output
+        self.assertResult([i1])
         
     
     
     def resolveCode(self,skip = False):
         solver = YumBase()
+        solver.save_ts  =  save_ts
         solver.arch.setup_arch('x86_64')
         solver.conf = FakeConf()
         solver.conf.skip_broken = skip
