@@ -14,8 +14,10 @@ from yum import packages
 from yum import packageSack
 from yum.constants import TS_INSTALL_STATES, TS_REMOVE_STATES
 from cli import YumBaseCli
+from yum.rpmsack import RPMDBPackageSack as _rpmdbsack
 import inspect
 from rpmUtils import arch
+from rpmUtils.transaction import initReadOnlyTransaction
 
 #############################################################
 ### Helper classes ##########################################
@@ -25,6 +27,10 @@ from rpmUtils import arch
 def _(msg):
     return msg
 
+# dummy save_ts to avoid lots of errors
+def save_ts(*args, **kwargs):
+    pass
+
 class FakeConf(object):
 
     def __init__(self):
@@ -43,11 +49,26 @@ class FakeConf(object):
         self.persistdir = '/should-not-exist-bad-test!'
         self.showdupesfromrepos = False
         self.uid = 0
+        self.groupremove_leaf_only = False
+        self.protected_packages = []
+        self.protected_multilib = False
+        self.clean_requirements_on_remove = True
+
+class FakeSack:
+    """ Fake PackageSack to use with FakeRepository"""
+    def __init__(self):
+        pass # This is fake, so do nothing
+    
+    def have_fastReturnFileEntries(self):
+        return True
 
 class FakeRepo(object):
 
+    __fake_sack = FakeSack()
     def __init__(self, id=None,sack=None):
         self.id = id
+        if sack is None:
+            sack = self.__fake_sack
         self.sack = sack
         self.cost = 1000
 
@@ -97,6 +118,10 @@ class FakePackage(packages.YumAvailablePackage):
         self.prco['obsoletes'].append((name, flag, evr))
     def addFile(self, name, ftype='file'):
         self.files[ftype].append(name)
+    def required_packages(self):
+        return []
+    def requiring_packages(self):
+        return []
 
 class _Container(object):
     pass
@@ -112,13 +137,17 @@ class DepSolveProgressCallBack:
     
     def pkgAdded(self, pkgtup, mode):
         modedict = { 'i': _('installed'),
-                     'u': _('updated'),
-                     'o': _('obsoleted'),
-                     'e': _('erased')}
+                     'u': _('an update'),
+                     'e': _('erased'),
+                     'r': _('reinstalled'),
+                     'd': _('a downgrade'),
+                     'o': _('obsoleting'),
+                     'ud': _('updated'),
+                     'od': _('obsoleted'),}
         (n, a, e, v, r) = pkgtup
         modeterm = modedict[mode]
         self.verbose_logger.log(logginglevels.INFO_2,
-            _('---> Package %s.%s %s:%s-%s set to be %s'), n, a, e, v, r,
+            _('---> Package %s.%s %s:%s-%s will be %s'), n, a, e, v, r,
             modeterm)
         
     def start(self):
@@ -233,6 +262,57 @@ class FakeRpmDb(packageSack.PackageSack):
     def __init__(self):
         packageSack.PackageSack.__init__(self)
 
+    # Need to mock out rpmdb caching... copy&paste. Gack.
+    def returnConflictPackages(self):
+        ret = []
+        for pkg in self.returnPackages():
+            if len(pkg.conflicts):
+                ret.append(pkg)
+        return ret
+    def fileRequiresData(self):
+        installedFileRequires = {}
+        installedUnresolvedFileRequires = set()
+        resolved = set()
+        for pkg in self.returnPackages():
+            for name, flag, evr in pkg.requires:
+                if not name.startswith('/'):
+                    continue
+                installedFileRequires.setdefault(pkg.pkgtup, []).append(name)
+                if name not in resolved:
+                    dep = self.getProvides(name, flag, evr)
+                    resolved.add(name)
+                    if not dep:
+                        installedUnresolvedFileRequires.add(name)
+
+        fileRequires = set()
+        for fnames in installedFileRequires.itervalues():
+            fileRequires.update(fnames)
+        installedFileProviders = {}
+        for fname in fileRequires:
+            pkgtups = [pkg.pkgtup for pkg in self.getProvides(fname)]
+            installedFileProviders[fname] = pkgtups
+
+        ret =  (installedFileRequires, installedUnresolvedFileRequires,
+                installedFileProviders)
+
+        return ret
+    def transactionCacheFileRequires(self, installedFileRequires,
+                                     installedUnresolvedFileRequires,
+                                     installedFileProvides,
+                                     problems):
+        return
+    def transactionCacheConflictPackages(self, pkgs):
+        return
+    def transactionResultVersion(self, rpmdbv):
+        return
+    def transactionReset(self):
+        return
+
+    def readOnlyTS(self):
+        #  Should probably be able to "fake" this, so we can provide different
+        # get_running_kernel_pkgtup(). Bah.
+        return initReadOnlyTransaction("/")
+
     def getProvides(self, name, flags=None, version=(None, None, None)):
         """return dict { packages -> list of matching provides }"""
         self._checkIndexes(failure='build')
@@ -290,9 +370,9 @@ class DepsolveTests(_DepsolveTestsBase):
     def resetTsInfo(self):
         self.tsInfo = transactioninfo.TransactionData()
         
-
     def resolveCode(self):
         solver = YumBase()
+        solver.save_ts  = save_ts
         solver.conf = FakeConf()
         solver.arch.setup_arch('x86_64')
         solver.tsInfo = solver._tsInfo = self.tsInfo
@@ -332,7 +412,8 @@ class OperationsTests(_DepsolveTestsBase):
     buildPkg code.
     """
 
-    def runOperation(self, args, installed=[], available=[]):
+    def runOperation(self, args, installed=[], available=[],
+                     confs={}, multi_cmds=False):
         """Sets up and runs the depsolver. args[0] must be a valid yum command
         ("install", "update", ...). It might be followed by pkg names as on the
         yum command line. The pkg objects in installed are added to self.rpmdb and
@@ -340,11 +421,14 @@ class OperationsTests(_DepsolveTestsBase):
         requirements from.
         """
         depsolver = YumBaseCli()
+        depsolver.save_ts = save_ts
         depsolver.arch.setup_arch('x86_64')
         self.rpmdb = depsolver.rpmdb = FakeRpmDb()
         self.xsack = depsolver._pkgSack  = packageSack.PackageSack()
         self.repo = depsolver.repo = FakeRepo("installed")
         depsolver.conf = FakeConf()
+        for conf in confs:
+            setattr(depsolver.conf, conf, confs[conf])
         # We are running nosetest, so we want to see some yum output
         # if a testcase if failing
         depsolver.doLoggingSetup(9,9)
@@ -359,9 +443,18 @@ class OperationsTests(_DepsolveTestsBase):
             po.repoid = po.repo.id
             self.depsolver._pkgSack.addPackage(po)
 
-        self.depsolver.basecmd = args[0]
-        self.depsolver.extcmds = args[1:]
-        res, msg = self.depsolver.doCommands()
+        if not multi_cmds:
+            self.depsolver.basecmd = args[0]
+            self.depsolver.extcmds = args[1:]
+            res, msg = self.depsolver.doCommands()
+        else:
+            for nargs in args:
+                self.depsolver.basecmd = nargs[0]
+                self.depsolver.extcmds = nargs[1:]
+                res, msg = self.depsolver.doCommands()
+                if res != 2:
+                    return res, msg
+
         self.tsInfo = depsolver.tsInfo
         if res!=2:
             return res, msg
