@@ -645,6 +645,8 @@ class YumConf(StartupConf):
     proxy = UrlOption(schemes=('http', 'ftp', 'https'), allow_none=True)
     proxy_username = Option()
     proxy_password = Option()
+    username = Option()
+    password = Option()
     installonlypkgs = ListOption(['kernel', 'kernel-bigmem',
             'kernel-enterprise','kernel-smp', 'kernel-modules', 'kernel-debug',
             'kernel-unsupported', 'kernel-source', 'kernel-devel', 'kernel-PAE',
@@ -671,6 +673,7 @@ class YumConf(StartupConf):
     # FIXME: rename gpgcheck to pkgs_gpgcheck
     gpgcheck = BoolOption(__pkgs_gpgcheck_default__)
     repo_gpgcheck = BoolOption(__repo_gpgcheck_default__)
+    localpkg_gpgcheck = BoolOption(__pkgs_gpgcheck_default__)
     obsoletes = BoolOption(True)
     showdupesfromrepos = BoolOption(False)
     enabled = BoolOption(True)
@@ -735,8 +738,13 @@ class YumConf(StartupConf):
 
     protected_packages = ListOption("yum, glob:/etc/yum/protected.d/*.conf",
                                     parse_default=True)
+    protected_multilib = BoolOption(True)
     exit_on_lock = BoolOption(False)
     
+    loadts_ignoremissing = BoolOption(False)
+    loadts_ignorerpm = BoolOption(False)
+    
+    clean_requirements_on_remove = BoolOption(False)
     _reposlist = []
 
     def dump(self):
@@ -786,6 +794,7 @@ class RepoConf(BaseConfig):
     metalink   = UrlOption()
     mediaid = Option()
     gpgkey = UrlListOption()
+    gpgcakey = UrlListOption()
     exclude = ListOption() 
     includepkgs = ListOption() 
 
@@ -794,6 +803,8 @@ class RepoConf(BaseConfig):
     proxy_password = Inherit(YumConf.proxy_password)
     retries = Inherit(YumConf.retries)
     failovermethod = Inherit(YumConf.failovermethod)
+    username = Inherit(YumConf.username)
+    password = Inherit(YumConf.password)
 
     # FIXME: rename gpgcheck to pkgs_gpgcheck
     gpgcheck = Inherit(YumConf.gpgcheck)
