@@ -27,6 +27,7 @@ import rpmUtils.miscutils
 from rpmUtils.arch import archDifference, canCoinstall
 import misc
 from misc import unique, version_tuple_to_string
+from transactioninfo import TransactionMember
 import rpm
 
 from packageSack import ListPackageSack
@@ -37,7 +38,7 @@ import Errors
 import warnings
 warnings.simplefilter("ignore", Errors.YumFutureDeprecationWarning)
 
-from yum import _
+from yum import _, _rpm_ver_atleast
 
 try:
     assert max(2, 4) == 4
@@ -65,7 +66,6 @@ class Depsolve(object):
     """
 
     def __init__(self):
-        packages.base = self
         self._ts = None
         self._tsInfo = None
         self.dsCallback = None
@@ -138,11 +138,14 @@ class Depsolve(object):
                             'test': rpm.RPMTRANS_FLAG_TEST,
                             'justdb': rpm.RPMTRANS_FLAG_JUSTDB,
                             'repackage': rpm.RPMTRANS_FLAG_REPACKAGE}
+        # This is only in newer rpm.org releases
+        if hasattr(rpm, 'RPMTRANS_FLAG_NOCONTEXTS'):
+            ts_flags_to_rpm['nocontexts'] = rpm.RPMTRANS_FLAG_NOCONTEXTS
         
         self._ts.setFlags(0) # reset everything.
         
         for flag in self.conf.tsflags:
-            if ts_flags_to_rpm.has_key(flag):
+            if flag in ts_flags_to_rpm:
                 self._ts.addTsFlag(ts_flags_to_rpm[flag])
             else:
                 self.logger.critical(_('Invalid tsflag in config file: %s'), flag)
@@ -158,43 +161,21 @@ class Depsolve(object):
 
         self.verbose_logger.log(logginglevels.DEBUG_1, _('Searching pkgSack for dep: %s'),
             name)
-        pkgs = self.pkgSack.searchProvides(name)
-        
-        
-        if flags == 0:
-            flags = None
-        if type(version) in (types.StringType, types.NoneType, types.UnicodeType):
-            (r_e, r_v, r_r) = rpmUtils.miscutils.stringToVersion(version)
-        elif type(version) in (types.TupleType, types.ListType): # would this ever be a ListType?
-            (r_e, r_v, r_r) = version
-        
-        defSack = ListPackageSack() # holder for items definitely providing this dep
-        
-        for po in pkgs:
-            self.verbose_logger.log(logginglevels.DEBUG_2,
-                _('Potential match for %s from %s'), name, po)
-            if misc.re_filename(name) and r_v is None:
-                # file dep add all matches to the defSack
-                defSack.addPackage(po)
-                continue
-
-            if po.checkPrco('provides', (name, flags, (r_e, r_v, r_r))):
-                defSack.addPackage(po)
-                self.verbose_logger.debug(_('Matched %s to require for %s'), po, name)
-        
+        defSack = ListPackageSack(self.pkgSack.searchProvides((name, flags, version)))
         return defSack
         
     def allowedMultipleInstalls(self, po):
         """takes a packageObject, returns 1 or 0 depending on if the package 
            should/can be installed multiple times with different vers
            like kernels and kernel modules, for example"""
-           
-        if po.name in self.conf.installonlypkgs:
+
+        iopkgs = set(self.conf.installonlypkgs)
+        if po.name in iopkgs:
             return True
         
-        provides = po.provides_names
-        if filter (lambda prov: prov in self.conf.installonlypkgs, provides):
-            return True
+        for prov in po.provides_names:
+            if prov in iopkgs:
+                return True
         
         return False
 
@@ -223,7 +204,7 @@ class Depsolve(object):
         for txmbr in self.tsInfo.getMembers():
             self.verbose_logger.log(logginglevels.DEBUG_3, _('Member: %s'), txmbr)
             if txmbr.ts_state in ['u', 'i']:
-                if ts_elem.has_key((txmbr.pkgtup, 'i')):
+                if (txmbr.pkgtup, 'i') in ts_elem:
                     continue
                 rpmfile = txmbr.po.localPkg()
                 if os.path.exists(rpmfile):
@@ -243,14 +224,20 @@ class Depsolve(object):
                 self.ts.addInstall(hdr, (hdr, rpmfile), txmbr.ts_state)
                 self.verbose_logger.log(logginglevels.DEBUG_1,
                     _('Adding Package %s in mode %s'), txmbr.po, txmbr.ts_state)
-                if self.dsCallback: 
-                    self.dsCallback.pkgAdded(txmbr.pkgtup, txmbr.ts_state)
+                if self.dsCallback:
+                    dscb_ts_state = txmbr.ts_state
+                    if dscb_ts_state == 'u' and txmbr.downgrades:
+                        dscb_ts_state = 'd'
+                    self.dsCallback.pkgAdded(txmbr.pkgtup, dscb_ts_state)
             
             elif txmbr.ts_state in ['e']:
-                if ts_elem.has_key((txmbr.pkgtup, txmbr.ts_state)):
+                if (txmbr.pkgtup, txmbr.ts_state) in ts_elem:
                     continue
                 self.ts.addErase(txmbr.po.idx)
-                if self.dsCallback: self.dsCallback.pkgAdded(txmbr.pkgtup, 'e')
+                if self.dsCallback:
+                    if txmbr.downgraded_by:
+                        continue
+                    self.dsCallback.pkgAdded(txmbr.pkgtup, 'e')
                 self.verbose_logger.log(logginglevels.DEBUG_1,
                     _('Removing Package %s'), txmbr.po)
 
@@ -307,6 +294,16 @@ class Depsolve(object):
     def _prco_req2req(self, req):
         return self._prco_req_nfv2req(req[0], req[1], req[2])
             
+    def _err_missing_requires(self, reqPo, reqTup):
+        if hasattr(self.dsCallback, 'format_missing_requires'):
+            msg = self.dsCallback.format_missing_requires(reqPo, reqTup)
+            if msg is not None: # PK
+                return self.dsCallback.format_missing_requires(reqPo, reqTup)
+        (needname, needflags, needversion) = reqTup
+        ui_req = rpmUtils.miscutils.formatRequire(needname, needversion,
+                                                  needflags)
+        return _('%s requires %s') % (reqPo, ui_req)
+
     def _requiringFromInstalled(self, requiringPo, requirement, errorlist):
         """processes the dependency resolution for a dep where the requiring 
            package is installed"""
@@ -331,7 +328,7 @@ class Depsolve(object):
         needpo = None
         providers = []
         
-        if self.cheaterlookup.has_key((needname, needflags, needversion)):
+        if (needname, needflags, needversion) in self.cheaterlookup:
             self.verbose_logger.log(logginglevels.DEBUG_2, _('Needed Require has already been looked up, cheating'))
             cheater_po = self.cheaterlookup[(needname, needflags, needversion)]
             providers = [cheater_po]
@@ -379,19 +376,47 @@ class Depsolve(object):
         self.verbose_logger.log(logginglevels.DEBUG_2, _('Mode for pkg providing %s: %s'), 
             niceformatneed, needmode)
 
+        if needmode in ['ud']: # the thing it needs is being updated or obsoleted away 
+            # try to update the requiring package in hopes that all this problem goes away :(
+            self.verbose_logger.log(logginglevels.DEBUG_2, _('Trying to update %s to resolve dep'), requiringPo)
+            # if the required pkg was updated, not obsoleted, then try to
+            # only update the requiring po
+            origobs = self.conf.obsoletes
+            self.conf.obsoletes = 0
+            txmbrs = self.update(po=requiringPo, requiringPo=requiringPo)
+            self.conf.obsoletes = origobs
+            if not txmbrs:
+                txmbrs = self.update(po=requiringPo, requiringPo=requiringPo)
+                if not txmbrs:
+                    msg = self._err_missing_requires(requiringPo, requirement)
+                    self.verbose_logger.log(logginglevels.DEBUG_2, _('No update paths found for %s. Failure!'), requiringPo)
+                    return self._requiringFromTransaction(requiringPo, requirement, errorlist)
+            checkdeps = 1
+
+        if needmode in ['od']: # the thing it needs is being updated or obsoleted away 
+            # try to update the requiring package in hopes that all this problem goes away :(
+            self.verbose_logger.log(logginglevels.DEBUG_2, _('Trying to update %s to resolve dep'), requiringPo)
+            txmbrs = self.update(po=requiringPo, requiringPo=requiringPo)
+            if not txmbrs:
+                msg = self._err_missing_requires(requiringPo, requirement)
+                self.verbose_logger.log(logginglevels.DEBUG_2, _('No update paths found for %s. Failure!'), requiringPo)
+                return self._requiringFromTransaction(requiringPo, requirement, errorlist)
+            checkdeps = 1
+
+            
         if needmode in ['e']:
             self.verbose_logger.log(logginglevels.DEBUG_2, _('TSINFO: %s package requiring %s marked as erase'),
                 requiringPo, needname)
-            txmbr = self.tsInfo.addErase(requiringPo)
-            txmbr.setAsDep(po=inst_po)
+            txmbrs = self.remove(po=requiringPo)
+            for txmbr in txmbrs:
+                txmbr.setAsDep(po=inst_po)
             checkdeps = 1
         
         if needmode in ['i', 'u']:
-            length = len(self.tsInfo)
-            self.update(name=name, epoch=epoch, version=ver, release=rel,
-                        requiringPo=requiringPo)
+            newupdates = self.update(name=name, epoch=epoch, version=ver, release=rel,
+                                     requiringPo=requiringPo)
             txmbrs = self.tsInfo.getMembersWithState(requiringPo.pkgtup, TS_REMOVE_STATES)
-            if len(self.tsInfo) != length and txmbrs:
+            if newupdates and txmbrs:
                 if txmbrs[0].output_state == TS_OBSOLETED:
                     self.verbose_logger.log(logginglevels.DEBUG_2, _('TSINFO: Obsoleting %s with %s to resolve dep.'),
                                             requiringPo, txmbrs[0].obsoleted_by[0])
@@ -413,9 +438,7 @@ class Depsolve(object):
             if self.pkgSack is None:
                 return self._requiringFromTransaction(requiringPo, requirement, errorlist)
             else:
-                prob_pkg = "%s (%s)" % (requiringPo,requiringPo.repoid)
-                msg = _('Unresolvable requirement %s for %s') % (niceformatneed,
-                                                               prob_pkg)
+                msg = self._err_missing_requires(requiringPo, requirement)
                 self.verbose_logger.log(logginglevels.DEBUG_2, msg)
                 checkdeps = 0
                 missingdep = 1
@@ -513,10 +536,7 @@ class Depsolve(object):
 
         if len(provSack) == 0: # unresolveable
             missingdep = 1
-            prob_pkg = "%s (%s)" % (requiringPo,requiringPo.repoid)
-            msg = _('Missing Dependency: %s is needed by package %s') % \
-            (rpmUtils.miscutils.formatRequire(needname, needversion, needflags),
-                                                                   prob_pkg)
+            msg = self._err_missing_requires(requiringPo, requirement)
             errorlist.append(msg)
             return checkdeps, missingdep
         
@@ -528,18 +548,26 @@ class Depsolve(object):
             if pkgmode in ['i', 'u']:
                 self.verbose_logger.log(logginglevels.DEBUG_2,
                     _('%s already in ts, skipping this one'), pkg)
-                # FIXME: Remove this line, if it is not needed ?
-                # checkdeps = 1
                 self._last_req = pkg
                 return checkdeps, missingdep
 
         # find the best one 
 
         # try updating the already install pkgs
+        results = []
         for pkg in provSack.returnNewestByName():
-            results = self.update(requiringPo=requiringPo, name=pkg.name,
-                                  epoch=pkg.epoch, version=pkg.version,
-                                  rel=pkg.rel)
+            tresults = self.update(requiringPo=requiringPo, name=pkg.name,
+                                   epoch=pkg.epoch, version=pkg.version,
+                                   rel=pkg.rel)
+            #  Note that this does "interesting" things with multilib. We can
+            # have say A.i686 and A.x86_64, and if we hit "A.i686" first,
+            # .update() will actually update "A.x86_64" which will then fail
+            # the pkg == txmbr.po test below, but then they'll be nothing to
+            # update when we get around to A.x86_64 ... so this entire loop
+            # fails.
+            #  Keeping results through the loop and thus. testing each pkg
+            # against all txmbr's from previous runs "fixes" this.
+            results.extend(tresults)
             for txmbr in results:
                 if pkg == txmbr.po:
                     checkdeps = True
@@ -559,13 +587,11 @@ class Depsolve(object):
         if self.rpmdb.contains(po=best): # is it already installed?
             missingdep = 1
             checkdeps = 0
-            prob_pkg = "%s (%s)" % (requiringPo,requiringPo.repoid)
-            msg = _('Missing Dependency: %s is needed by package %s') % (needname, prob_pkg)
+            msg = self._err_missing_requires(requiringPo, requirement)
             errorlist.append(msg)
             return checkdeps, missingdep
         
                 
-            
         # FIXME - why can't we look up in the transaction set for the requiringPkg
         # and know what needs it that way and provide a more sensible dep structure in the txmbr
         inst = self.rpmdb.searchNevra(name=best.name, arch=best.arch)
@@ -576,23 +602,30 @@ class Depsolve(object):
             txmbr = self.tsInfo.addUpdate(best, inst[0])
             txmbr.setAsDep(po=requiringPo)
             txmbr.reason = "dep"
+            checkdeps = True
             self._last_req = best
         else:
             self.verbose_logger.debug(_('TSINFO: Marking %s as install for %s'), best,
                 requiringPo)
-            # FIXME: Don't we want .install() here, so obsoletes get done?
-            txmbr = self.tsInfo.addInstall(best)
-            txmbr.setAsDep(po=requiringPo)
-            txmbr.reason = "dep"
-            self._last_req = best
+            reqtuple = misc.string_to_prco_tuple(needname + str(needflags) + needversion)
+            txmbrs = self.install(best, provides_for=reqtuple)
+            for txmbr in txmbrs:
+                txmbr.setAsDep(po=requiringPo)
+                txmbr.reason = "dep"
+                self._last_req = txmbr.po
 
-            # if we had other packages with this name.arch that we found
-            # before, they're not going to be installed anymore, so we
-            # should mark them to be re-checked
-            if upgraded.has_key(best.pkgtup):
-                map(self.tsInfo.remove, upgraded[best.pkgtup])
-
-        checkdeps = 1
+                # if we had other packages with this name.arch that we found
+                # before, they're not going to be installed anymore, so we
+                # should mark them to be re-checked
+                if txmbr.pkgtup in upgraded:
+                    map(self.tsInfo.remove, upgraded[txmbr.pkgtup])
+            if not txmbrs:
+                missingdep = 1
+                checkdeps = 0
+                msg = self._err_missing_requires(requiringPo, requirement)
+                errorlist.append(msg)
+            else:
+                checkdeps = 1
         
         return checkdeps, missingdep
 
@@ -757,15 +790,24 @@ class Depsolve(object):
         self.tsInfo.changed = False
         if len(errors) > 0:
             errors = unique(errors)
+            #  We immediately display this in cli, so don't show it twice.
+            # Plus skip-broken can get here N times. Might be worth keeping
+            # around for debugging?
+            done = set() # Same as the unique above
             for po,wpo,err in self.po_with_problems:
-                self.verbose_logger.info(_("%s from %s has depsolving problems") % (po,po.repoid))
-                self.verbose_logger.info("  --> %s" % (err))
+                if (po,err) in done:
+                    continue
+                done.add((po, err))
+                self.verbose_logger.log(logginglevels.DEBUG_4,
+                    _("%s from %s has depsolving problems") % (po, po.repoid))
+                err = err.replace('\n', '\n  --> ')
+                self.verbose_logger.log(logginglevels.DEBUG_4,"  --> %s" % err)
             return (1, errors)
 
-        if len(self.tsInfo) > 0:
-            if not len(self.tsInfo):
-                return (0, [_('Success - empty transaction')])
-            return (2, [_('Success - deps resolved')])
+        if not len(self.tsInfo):
+            return (0, [_('Success - empty transaction')])
+        
+        return (2, [_('Success - deps resolved')])
 
     def _resolveRequires(self, errors):
         any_missing = False
@@ -777,7 +819,17 @@ class Depsolve(object):
         for txmbr in self.tsInfo.getUnresolvedMembers():
 
             if self.dsCallback and txmbr.ts_state:
-                self.dsCallback.pkgAdded(txmbr.pkgtup, txmbr.ts_state)
+                dscb_ts_state = txmbr.ts_state
+                if txmbr.downgrades:
+                    dscb_ts_state = 'd'
+                if dscb_ts_state == 'u' and txmbr.reinstall:
+                    dscb_ts_state = 'r'
+                if dscb_ts_state == 'u':
+                    if txmbr.output_state == TS_OBSOLETING:
+                        dscb_ts_state = 'o'
+                    elif not txmbr.updates:
+                        dscb_ts_state = 'i'
+                self.dsCallback.pkgAdded(txmbr.pkgtup, dscb_ts_state)
             self.verbose_logger.log(logginglevels.DEBUG_2,
                                     _("Checking deps for %s") %(txmbr,))
 
@@ -799,6 +851,14 @@ class Depsolve(object):
 
             missing_in_pkg = False
             for po, dep in thisneeds:
+                if txmbr.downgraded_by: # Don't try to chain remove downgrades
+                    msg = self._err_missing_requires(po, dep)
+                    self.verbose_logger.log(logginglevels.DEBUG_2, msg)
+                    errors.append(msg)
+                    self.po_with_problems.add((po,self._working_po,errors[-1]))
+                    missing_in_pkg = 1
+                    continue
+
                 (checkdep, missing, errormsgs) = self._processReq(po, dep)
                 CheckDeps |= checkdep
                 errors += errormsgs
@@ -812,18 +872,15 @@ class Depsolve(object):
         return CheckDeps, CheckInstalls, CheckRemoves, any_missing
 
     @staticmethod
-    def _sort_reqs(pkgtup1, pkgtup2):
-        """ Sort the requires for a package from most "narrow" to least,
+    def _sort_req_key(pkgtup):
+        """ Get a sort key for a package requires from most "narrow" to least,
             this tries to ensure that if we have two reqs like
             "libfoo = 1.2.3-4" and "foo-api" (which is also provided by
             libxyz-foo) that we'll get just libfoo.
             There are other similar cases this "handles"."""
 
-        mapper = {'EQ' : 1, 'LT' : 2, 'LE' : 3, 'GT' : 4, 'GE' : 5,
-                  None : 99}
-        ret = mapper.get(pkgtup1[1], 10) - mapper.get(pkgtup2[1], 10)
-        if ret:
-            return ret
+        mapper = {'EQ' : 1, 'LT' : 2, 'LE' : 3, 'GT' : 4, 'GE' : 5, None : 99}
+        flagscore = mapper.get(pkgtup[1], 10)
 
         # This is pretty magic, basically we want an explicit:
         #
@@ -835,16 +892,16 @@ class Depsolve(object):
         #
         # ...because sometimes the libfoo.so.0() is provided by multiple
         # packages. Do we need more magic for other implicit deps. here?
-        def _req_name2val(name):
-            if (name.startswith("lib") and
-                (name.endswith("()") or name.endswith("()(64bit)"))):
-                return 99 # Processes these last
-            return 0
-        return _req_name2val(pkgtup1[0]) - _req_name2val(pkgtup2[0])
+
+        namescore = 0
+        if pkgtup[0].startswith("lib") and \
+                (pkgtup[0].endswith("()") or pkgtup[0].endswith("()(64bit)")):
+            namescore = 99 # Processes these last
+
+        return (flagscore, namescore)
 
     def _checkInstall(self, txmbr):
         txmbr_reqs = txmbr.po.returnPrco('requires')
-        txmbr_provs = set(txmbr.po.returnPrco('provides'))
 
         # if this is an update, we should check what the old
         # requires were to make things faster
@@ -854,17 +911,19 @@ class Depsolve(object):
         oldreqs = set(oldreqs)
 
         ret = []
-        for req in sorted(txmbr_reqs, cmp=self._sort_reqs):
+        for req in sorted(txmbr_reqs, key=self._sort_req_key):
             if req[0].startswith('rpmlib('):
                 continue
-            if req in txmbr_provs:
-                continue
-            if req in oldreqs and self.rpmdb.getProvides(*req):
+            if req in oldreqs:
                 continue
             
             self.verbose_logger.log(logginglevels.DEBUG_2, _("looking for %s as a requirement of %s"), req, txmbr)
             provs = self.tsInfo.getProvides(*req)
-            if not provs:
+            #  The self provides should mostly be caught before here now, but
+            # at least config() crack still turns up, it's not that
+            # expensive to just do it, and we really don't want "false positive"
+            # requires for compare_providers().
+            if not provs and not txmbr.po.inPrcoRange('provides', req):
                 ret.append( (txmbr.po, self._prco_req2req(req)) )
                 continue
 
@@ -885,7 +944,7 @@ class Depsolve(object):
         # if this is an update, we should check what the new package
         # provides to make things faster
         newpoprovs = {}
-        for newpo in txmbr.updated_by:
+        for newpo in txmbr.updated_by + txmbr.obsoleted_by:
             for p in newpo.provides:
                 newpoprovs[p] = 1
         ret = []
@@ -895,43 +954,65 @@ class Depsolve(object):
         for prov in provs:
             if prov[0].startswith('rpmlib('): # ignore rpmlib() provides
                 continue
-            if newpoprovs.has_key(prov):
+            if prov in newpoprovs:
                 continue
             # FIXME: This is probably the best place to fix the postfix rename
             # problem long term (post .21) ... see compare_providers.
             for pkg, hits in self.tsInfo.getRequires(*prov).iteritems():
-                for rn, rf, rv in hits:
+                # See the docs, this is to make groupremove "more useful".
+                if (self.conf.groupremove_leaf_only and txmbr.groups and
+                    txmbr.output_state == TS_ERASE):
+                    cb = self.dsCallback
+                    if cb and hasattr(cb, 'groupRemoveReq'):
+                        cb.groupRemoveReq(pkg, hits)
+                    #  We don't undo anything else here ... hopefully that's
+                    # fine.
+                    self.tsInfo.remove(txmbr.pkgtup)
+                    return []
+
+                for hit in hits:
+                    # See if the update solves the problem...
+                    found = False
+                    for newpo in txmbr.updated_by:
+                        if newpo.checkPrco('provides', hit):
+                            found = True
+                            break
+                    if found: continue
+                    for newpo in txmbr.obsoleted_by:
+                        if newpo.checkPrco('provides', hit):
+                            found = True
+                            break
+                    if found: continue
+
+                    # It doesn't, so see what else might...
+                    rn, rf, rv = hit
                     if not self.tsInfo.getProvides(rn, rf, rv):
                         ret.append( (pkg, self._prco_req_nfv2req(rn, rf, rv)) )
         return ret
 
     def _checkFileRequires(self):
         fileRequires = set()
+        nfileRequires = set() # These need to be looked up in the rpmdb.
         reverselookup = {}
         ret = []
 
         # generate list of file requirement in rpmdb
         if self.installedFileRequires is None:
-            self.installedFileRequires = {}
-            self.installedUnresolvedFileRequires = set()
-            resolved = set()
-            for pkg in self.rpmdb.returnPackages():
-                for name, flag, evr in pkg.requires:
-                    if not name.startswith('/'):
-                        continue
-                    self.installedFileRequires.setdefault(pkg, []).append(name)
-                    if name not in resolved:
-                        dep = self.rpmdb.getProvides(name, flag, evr)
-                        resolved.add(name)
-                        if not dep:
-                            self.installedUnresolvedFileRequires.add(name)
+            self.installedFileRequires, \
+              self.installedUnresolvedFileRequires, \
+              self.installedFileProviders = self.rpmdb.fileRequiresData()
 
         # get file requirements from packages not deleted
-        for po, files in self.installedFileRequires.iteritems():
-            if not self._tsInfo.getMembersWithState(po.pkgtup, output_states=TS_REMOVE_STATES):
+        todel = []
+        for pkgtup, files in self.installedFileRequires.iteritems():
+            if self._tsInfo.getMembersWithState(pkgtup, output_states=TS_REMOVE_STATES):
+                todel.append(pkgtup)
+            else:
                 fileRequires.update(files)
                 for filename in files:
-                    reverselookup.setdefault(filename, []).append(po)
+                    reverselookup.setdefault(filename, []).append(pkgtup)
+        for pkgtup in todel:
+            del self.installedFileRequires[pkgtup]
 
         fileRequires -= self.installedUnresolvedFileRequires
 
@@ -939,6 +1020,8 @@ class Depsolve(object):
         for txmbr in self._tsInfo.getMembersWithState(output_states=TS_INSTALL_STATES):
             for name, flag, evr in txmbr.po.requires:
                 if name.startswith('/'):
+                    pt = txmbr.po.pkgtup
+                    self.installedFileRequires.setdefault(pt, []).append(name)
                     # check if file requires was already unresolved in update
                     if name in self.installedUnresolvedFileRequires:
                         already_broken = False
@@ -948,24 +1031,83 @@ class Depsolve(object):
                                 break
                         if already_broken:
                             continue
+                    if name not in fileRequires:
+                        nfileRequires.add(name)
                     fileRequires.add(name)
-                    reverselookup.setdefault(name, []).append(txmbr.po)
+                    reverselookup.setdefault(name, []).append(txmbr.po.pkgtup)
+
+        todel = []
+        for fname in self.installedFileProviders:
+            niFP_fname = []
+            for pkgtup in self.installedFileProviders[fname]:
+                if self._tsInfo.getMembersWithState(pkgtup, output_states=TS_REMOVE_STATES):
+                    continue
+                niFP_fname.append(pkgtup)
+
+            if not niFP_fname:
+                todel.append(fname)
+                continue
+
+            self.installedFileProviders[fname] = niFP_fname
+        for fname in todel:
+            del self.installedFileProviders[fname]
 
         # check the file requires
+        iFP = self.installedFileProviders
         for filename in fileRequires:
-            if not self.tsInfo.getOldProvides(filename) and not self.tsInfo.getNewProvides(filename):
-                for po in reverselookup[filename]:
-                    ret.append( (po, (filename, 0, '')) )
+            # In theory we need this to be:
+            #
+            # nprov, filename in iFP (or new), oprov
+            #
+            # ...this keeps the cache exactly the same as the non-cached data.
+            # However that also means that we'll always need the filelists, so
+            # we do:
+            #
+            # filename in iFP (if found return), oprov (if found return),
+            # nprov
+            #
+            # ...this means we'll always get the same _result_ (as we only need
+            # to know if _something_ provides), but our cache will be off on
+            # what does/doesn't provide the file.
+            if filename in self.installedFileProviders:
+                continue
+
+            oprov = self.tsInfo.getOldProvides(filename)
+            if oprov:
+                iFP.setdefault(filename, []).extend([po.pkgtup for po in oprov])
+                continue
+
+            nprov = self.tsInfo.getNewProvides(filename)
+            if nprov:
+                iFP.setdefault(filename, []).extend([po.pkgtup for po in nprov])
+                continue 
+
+            for pkgtup in reverselookup[filename]:
+                po = self.tsInfo.getMembersWithState(pkgtup, TS_INSTALL_STATES)
+                if po:
+                    po = po[0].po # Should only have one
+                else:
+                    po = self.getInstalledPackageObject(pkgtup)
+                ret.append( (po, (filename, 0, '')) )
+
+        self.rpmdb.transactionCacheFileRequires(self.installedFileRequires, 
+                                        self.installedUnresolvedFileRequires,
+                                        self.installedFileProviders,
+                                        ret)
 
         return ret
 
-
     def _checkConflicts(self):
         ret = [ ]
-        for po in self.rpmdb.returnPackages():
+        cpkgs = []
+        for po in self.rpmdb.returnConflictPackages():
             if self.tsInfo.getMembersWithState(po.pkgtup, output_states=TS_REMOVE_STATES):
                 continue
-            for conflict in po.returnPrco('conflicts'):
+            conflicts = po.returnPrco('conflicts')
+            if not conflicts: # We broke this due to dbMatch() usage.
+                continue
+            cpkgs.append(po)
+            for conflict in conflicts:
                 (r, f, v) = conflict
                 for conflicting_po in self.tsInfo.getNewProvides(r, f, v):
                     if conflicting_po.pkgtup[0] == po.pkgtup[0] and conflicting_po.pkgtup[2:] == po.pkgtup[2:]:
@@ -974,30 +1116,39 @@ class Depsolve(object):
                                  conflicting_po) )
         for txmbr in self.tsInfo.getMembersWithState(output_states=TS_INSTALL_STATES):
             po = txmbr.po
+            done = False
             for conflict in txmbr.po.returnPrco('conflicts'):
+                if not done:
+                    cpkgs.append(txmbr.po)
+                    done = True
                 (r, f, v) = conflict
                 for conflicting_po in self.tsInfo.getProvides(r, f, v):
                     if conflicting_po.pkgtup[0] == po.pkgtup[0] and conflicting_po.pkgtup[2:] == po.pkgtup[2:]:
                         continue
                     ret.append( (po, self._prco_req_nfv2req(r, f, v),
                                  conflicting_po) )
+
+        if _rpm_ver_atleast((4, 9, 0)):
+            return ret # Don't need the conflicts cache anymore
+
+        self.rpmdb.transactionCacheConflictPackages(cpkgs)
         return ret
 
-
     def isPackageInstalled(self, pkgname):
-        installed = False
-        if self.rpmdb.contains(name=pkgname):
-            installed = True
-
         lst = self.tsInfo.matchNaevr(name = pkgname)
         for txmbr in lst:
             if txmbr.output_state in TS_INSTALL_STATES:
                 return True
-        if installed and len(lst) > 0:
-            # if we get here, then it was installed, but it's in the tsInfo
-            # for an erase or obsoleted --> not going to be installed at end
+
+        if len(lst) > 0:
+            # if we get here then it's in the tsInfo for an erase or obsoleted
+            #  --> not going to be installed
             return False
-        return installed
+
+        if not self.rpmdb.contains(name=pkgname):
+            return False
+
+        return True
     _isPackageInstalled = isPackageInstalled
 
     def _compare_providers(self, pkgs, reqpo):
@@ -1005,13 +1156,6 @@ class Depsolve(object):
            return a dictionary of po=score"""
         self.verbose_logger.log(logginglevels.DEBUG_4,
               _("Running compare_providers() for %s") %(str(pkgs)))
-
-        def _cmp_best_providers(x, y):
-            """ Compare first by score, and then compare the pkgs if the score
-                is the same. Note that this sorts in reverse. """
-            ret = cmp(y[1], x[1])
-            if ret: return ret
-            return cmp(y[0], x[0])
         
         def _common_prefix_len(x, y, minlen=2):
             num = min(len(x), len(y))
@@ -1063,11 +1207,15 @@ class Depsolve(object):
         pkgs = unique_nevra_pkgs.values()
             
         pkgresults = {}
-        ipkgresults = {}
 
         for pkg in pkgs:
             pkgresults[pkg] = 0
-
+        
+        # hand this off to our plugins
+        self.plugins.run("compare_providers", providers_dict=pkgresults, 
+                                      reqpo=reqpo)
+        
+        for pkg in pkgresults.keys():
             rpmdbpkgs = self.rpmdb.searchNevra(name=pkg.name)
             if rpmdbpkgs:
                 #  We only want to count things as "installed" if they are
@@ -1081,23 +1229,21 @@ class Depsolve(object):
                     # we are giving an edge to is not obsoleted by
                     # something else in the transaction. :(
                     # there are many ways I hate this - this is but one
-                    ipkgresults[pkg] = 5
+                    pkgresults[pkg] += 5
+                elif newest.verEQ(pkg):
+                    #  We get here from bestPackagesFromList(), give a giant
+                    # bump to stuff that is already installed.
+                    pkgresults[pkg] += 1000
+                elif newest.verGT(pkg):
+                    # if the version we're looking at is older than what we have installed
+                    # score it down like we would an obsoleted pkg
+                    pkgresults[pkg] -= 1024
             else:
                 # just b/c they're not installed pkgs doesn't mean they should
                 # be ignored entirely. Just not preferred
-                ipkgresults[pkg] = 0
+                pass
 
-        #  This is probably only for "renames". What happens is that pkgA-1 gets
-        # obsoleted by pkgB but pkgB requires pkgA-2, now _if_ the pkgA txmbr
-        # gets processed before pkgB then we'll process the "checkRemove" of
-        # pkgA ... so any deps. on pkgA-1 will look for a new provider, one of
-        # which is pkgA-2 in that case we want to choose that pkg over any
-        # others. This works for multiple cases too, but who'd do that right?:)
-        #  FIXME: A good long term. fix here is to just not get into this
-        # problem, but that's too much for .21. This is much safer.
-        if ipkgresults:
-            pkgresults = ipkgresults
-            pkgs = ipkgresults.keys()
+        pkgs = pkgresults.keys()
             
         # go through each pkg and compare to others
         # if it is same skip it
@@ -1133,8 +1279,7 @@ class Depsolve(object):
                     pkgresults[po] -= 1024
 
                 obsoleted = False
-                poprovtup = (po.name, 'EQ', (po.epoch, po.ver, po.release))
-                if nextpo.inPrcoRange('obsoletes', poprovtup):
+                if po.obsoletedBy([nextpo]):
                     obsoleted = True
                     pkgresults[po] -= 1024
                                 
@@ -1156,10 +1301,15 @@ class Depsolve(object):
                     if res == po:
                         pkgresults[po] += 5
 
+            # End of O(N*N): for nextpo in pkgs:
             if _common_sourcerpm(po, reqpo):
                 self.verbose_logger.log(logginglevels.DEBUG_4,
                     _('common sourcerpm %s and %s' % (po, reqpo)))
                 pkgresults[po] += 20
+            if self.isPackageInstalled(po.base_package_name):
+                self.verbose_logger.log(logginglevels.DEBUG_4,
+                    _('base package %s is installed for %s' % (po.base_package_name, po)))
+                pkgresults[po] += 5 # Same as before - - but off of base package name
             if reqpo:
                 cpl = _common_prefix_len(po.name, reqpo.name)
                 if cpl > 2:
@@ -1167,10 +1317,58 @@ class Depsolve(object):
                         _('common prefix of %s between %s and %s' % (cpl, po, reqpo)))
                 
                     pkgresults[po] += cpl*2
-            
+                
+        #  If we have more than one "best", see what would happen if we picked
+        # each package ... ie. what things do they require that _aren't_ already
+        # installed/to-be-installed. In theory this can screw up due to:
+        #   pkgA => requires pkgX
+        #   pkgB => requires pkgY, requires pkgZ
+        # ...but pkgX requires 666 other things. Going recursive is
+        # "non-trivial" though, python != prolog. This seems to do "better"
+        # from simple testing though.
+        bestnum = max(pkgresults.values())
+        rec_depsolve = {}
+        for po in pkgs:
+            if pkgresults[po] != bestnum:
+                continue
+            rec_depsolve[po] = 0
+        if len(rec_depsolve) > 1:
+            for po in rec_depsolve:
+                fake_txmbr = TransactionMember(po)
+
+                #  Note that this is just requirements, so you could also have
+                # 4 requires for a single package. This might be fixable, if
+                # needed, but given the above it's probably better to leave it
+                # like this.
+                reqs = self._checkInstall(fake_txmbr)
+                rec_depsolve[po] = len(reqs)
+
+            bestnum = min(rec_depsolve.values())
+            self.verbose_logger.log(logginglevels.DEBUG_4,
+                                    _('requires minimal: %d') % bestnum)
+            for po in rec_depsolve:
+                if rec_depsolve[po] == bestnum:
+                    self.verbose_logger.log(logginglevels.DEBUG_4,
+                            _(' Winner: %s') % po)
+                    pkgresults[po] += 1
+                else:
+                    num = rec_depsolve[po]
+                    self.verbose_logger.log(logginglevels.DEBUG_4,
+                            _(' Loser(with %d): %s') % (num, po))
+
+        #  We don't want to decide to use a "shortest first", if something else
+        # has told us to pick something else. But we want to pick between
+        # multiple "best" packages. So we spike all the best packages (so
+        # only those can win) and then bump them down by package name length.
+        bestnum = max(pkgresults.values())
+        for po in pkgs:
+            if pkgresults[po] != bestnum:
+                continue
+            pkgresults[po] += 1000
             pkgresults[po] += (len(po.name)*-1)
 
-        bestorder = sorted(pkgresults.items(), cmp=_cmp_best_providers)
+        bestorder = sorted(pkgresults.items(),
+                           key=lambda x: (x[1], x[0]), reverse=True)
         self.verbose_logger.log(logginglevels.DEBUG_4,
                 _('Best Order: %s' % str(bestorder)))
 
