@@ -32,9 +32,12 @@ class _wrap_ayum_getKeyForRepo:
         we have a seperate class).
         A "better" fix might be to explicitly pass the YumBase instance to
         the callback ... API change! """
-    def __init__(self, ayum):
+    def __init__(self, ayum, ca=False):
         self.ayum = weakref(ayum)
+        self.ca = ca
     def __call__(self, repo, callback=None):
+        if self.ca:
+            return self.ayum.getCAKeyForRepo(repo, callback)
         return self.ayum.getKeyForRepo(repo, callback)
 
 class RepoStorage:
@@ -57,6 +60,7 @@ class RepoStorage:
         # even quasi-useful
         # defaults to what is probably sane-ish
         self.gpg_import_func = _wrap_ayum_getKeyForRepo(ayum)
+        self.gpgca_import_func = _wrap_ayum_getKeyForRepo(ayum, ca=True)
         self.confirm_func = None
 
         # This allow listEnabled() to be O(1) most of the time.
@@ -77,8 +81,13 @@ class RepoStorage:
 
         for repo in repos:
             repo.setup(self.ayum.conf.cache, self.ayum.mediagrabber,
-                   gpg_import_func = self.gpg_import_func, confirm_func=self.confirm_func)
-
+                   gpg_import_func = self.gpg_import_func, confirm_func=self.confirm_func,
+                   gpgca_import_func = self.gpgca_import_func)
+            # if we come back from setup NOT enabled then mark as disabled
+            # so nothing else touches us
+            if not repo.enabled:
+                self.disableRepo(repo.id)
+                
         self._setup = True
         self.ayum.plugins.run('postreposetup')
         
@@ -93,7 +102,7 @@ class RepoStorage:
             repo.close()
 
     def add(self, repoobj):
-        if self.repos.has_key(repoobj.id):
+        if repoobj.id in self.repos:
             raise Errors.DuplicateRepoError, 'Repository %s is listed more than once in the configuration' % (repoobj.id)
         self.repos[repoobj.id] = repoobj
         if hasattr(repoobj, 'quick_enable_disable'):
@@ -101,9 +110,10 @@ class RepoStorage:
             repoobj.quick_enable_disable = self.quick_enable_disable
         else:
             self._cache_enabled_repos = None
+        repoobj._override_sigchecks = self.ayum._override_sigchecks
 
     def delete(self, repoid):
-        if self.repos.has_key(repoid):
+        if repoid in self.repos:
             thisrepo = self.repos[repoid]
             thisrepo.close()
             del self.repos[repoid]
@@ -205,11 +215,12 @@ class RepoStorage:
 
     def setCacheDir(self, cachedir):
         """sets the cachedir value in all repos"""
-
+        
+        self._cachedir = cachedir
         for repo in self.repos.values():
             repo.old_base_cache_dir = repo.basecachedir
             repo.basecachedir = cachedir
-            
+
 
     def setProgressBar(self, obj):
         """sets the progress bar for downloading files from repos"""
@@ -274,8 +285,15 @@ class RepoStorage:
          
         for repo in myrepos:
             sack = repo.getPackageSack()
-            sack.populate(repo, mdtype, callback, cacheonly)
-            self.pkgSack.addSack(repo.id, sack)
+            try:
+                sack.populate(repo, mdtype, callback, cacheonly)
+            except Errors.RepoError, e:
+                if mdtype in ['all', 'metadata'] and repo.skip_if_unavailable:
+                    self.disableRepo(repo.id)
+                else:
+                    raise
+            else:
+                self.pkgSack.addSack(repo.id, sack)
 
 
 class Repository:
