@@ -122,10 +122,30 @@ def checkShellArg(base, basecmd, extcmds):
         base.usage()
         raise cli.CliError
 
+def checkEnabledRepo(base, possible_local_files=[]):
+    """
+    Verify that there is at least one enabled repo.
+
+    @param base: a YumBase object.
+    """
+    if base.repos.listEnabled():
+        return
+
+    for lfile in possible_local_files:
+        if lfile.endswith(".rpm") and os.path.exists(lfile):
+            return
+
+    msg = _('There are no enabled repos.\n'
+            ' Run "yum repolist all" to see the repos you have.\n'
+            ' You can enable repos with yum-config-manager --enable <repo>')
+    base.logger.critical(msg)
+    raise cli.CliError
+
 class YumCommand:
         
     def __init__(self):
         self.done_command_once = False
+        self.hidden = False
 
     def doneCommand(self, base, msg, *args):
         if not self.done_command_once:
@@ -176,6 +196,7 @@ class InstallCommand(YumCommand):
         checkRootUID(base)
         checkGPGKey(base)
         checkPackageArg(base, basecmd, extcmds)
+        checkEnabledRepo(base, extcmds)
 
     def doCommand(self, base, basecmd, extcmds):
         self.doneCommand(base, _("Setting up Install Process"))
@@ -197,6 +218,7 @@ class UpdateCommand(YumCommand):
     def doCheck(self, base, basecmd, extcmds):
         checkRootUID(base)
         checkGPGKey(base)
+        checkEnabledRepo(base, extcmds)
 
     def doCommand(self, base, basecmd, extcmds):
         self.doneCommand(base, _("Setting up Update Process"))
@@ -218,6 +240,7 @@ class DistroSyncCommand(YumCommand):
     def doCheck(self, base, basecmd, extcmds):
         checkRootUID(base)
         checkGPGKey(base)
+        checkEnabledRepo(base, extcmds)
 
     def doCommand(self, base, basecmd, extcmds):
         self.doneCommand(base, _("Setting up Distribution Synchronization Process"))
@@ -420,6 +443,9 @@ class GroupListCommand(GroupCommand):
     def getSummary(self):
         return _("List available package groups")
     
+    def doCheck(self, base, basecmd, extcmds):
+        checkEnabledRepo(base)
+
     def doCommand(self, base, basecmd, extcmds):
         GroupCommand.doCommand(self, base, basecmd, extcmds)
         return base.returnGroupLists(extcmds)
@@ -441,6 +467,7 @@ class GroupInstallCommand(GroupCommand):
         checkRootUID(base)
         checkGPGKey(base)
         checkGroupArg(base, basecmd, extcmds)
+        checkEnabledRepo(base)
 
     def doCommand(self, base, basecmd, extcmds):
         GroupCommand.doCommand(self, base, basecmd, extcmds)
@@ -462,6 +489,7 @@ class GroupRemoveCommand(GroupCommand):
     def doCheck(self, base, basecmd, extcmds):
         checkRootUID(base)
         checkGroupArg(base, basecmd, extcmds)
+        checkEnabledRepo(base)
 
     def doCommand(self, base, basecmd, extcmds):
         GroupCommand.doCommand(self, base, basecmd, extcmds)
@@ -488,6 +516,7 @@ class GroupInfoCommand(GroupCommand):
 
     def doCheck(self, base, basecmd, extcmds):
         checkGroupArg(base, basecmd, extcmds)
+        checkEnabledRepo(base)
 
     def doCommand(self, base, basecmd, extcmds):
         GroupCommand.doCommand(self, base, basecmd, extcmds)
@@ -511,7 +540,7 @@ class MakeCacheCommand(YumCommand):
         return _("Generate the metadata cache")
 
     def doCheck(self, base, basecmd, extcmds):
-        pass
+        checkEnabledRepo(base)
 
     def doCommand(self, base, basecmd, extcmds):
         base.logger.debug(_("Making cache files for all metadata files."))
@@ -554,6 +583,7 @@ class CleanCommand(YumCommand):
 
     def doCheck(self, base, basecmd, extcmds):
         checkCleanArg(base, basecmd, extcmds)
+        checkEnabledRepo(base)
         
     def doCommand(self, base, basecmd, extcmds):
         base.conf.cache = 1
@@ -592,6 +622,9 @@ class CheckUpdateCommand(YumCommand):
     def getSummary(self):
         return _("Check for available package updates")
 
+    def doCheck(self, base, basecmd, extcmds):
+        checkEnabledRepo(base)
+
     def doCommand(self, base, basecmd, extcmds):
         base.extcmds.insert(0, 'updates')
         result = 0
@@ -668,6 +701,7 @@ class UpgradeCommand(YumCommand):
     def doCheck(self, base, basecmd, extcmds):
         checkRootUID(base)
         checkGPGKey(base)
+        checkEnabledRepo(base, extcmds)
 
     def doCommand(self, base, basecmd, extcmds):
         base.conf.obsoletes = 1
@@ -678,6 +712,10 @@ class UpgradeCommand(YumCommand):
             return 1, [str(e)]
 
 class LocalInstallCommand(YumCommand):
+    def __init__(self):
+        YumCommand.__init__(self)
+        self.hidden = True
+
     def getNames(self):
         return ['localinstall', 'localupdate']
 
@@ -1104,6 +1142,7 @@ class ReInstallCommand(YumCommand):
         checkRootUID(base)
         checkGPGKey(base)
         checkPackageArg(base, basecmd, extcmds)
+        checkEnabledRepo(base, extcmds)
 
     def doCommand(self, base, basecmd, extcmds):
         self.doneCommand(base, _("Setting up Reinstall Process"))
@@ -1130,6 +1169,7 @@ class DowngradeCommand(YumCommand):
         checkRootUID(base)
         checkGPGKey(base)
         checkPackageArg(base, basecmd, extcmds)
+        checkEnabledRepo(base, extcmds)
 
     def doCommand(self, base, basecmd, extcmds):
         self.doneCommand(base, _("Setting up Downgrade Process"))
@@ -1294,7 +1334,7 @@ class HistoryCommand(YumCommand):
         return ['history']
 
     def getUsage(self):
-        return "[info|list|summary|redo|undo|new]"
+        return "[info|list|summary|addon-info|package-list|redo|undo|new]"
 
     def getSummary(self):
         return _("Display, or use, the transaction history")
@@ -1323,7 +1363,10 @@ class HistoryCommand(YumCommand):
         base.history._create_db_file()
 
     def doCheck(self, base, basecmd, extcmds):
-        cmds = ('list', 'info', 'summary', 'repeat', 'redo', 'undo', 'new', 'addon-info')
+        cmds = ('list', 'info', 'summary', 'repeat', 'redo', 'undo', 'new',
+                'addon', 'addon-info',
+                'pkg', 'pkgs', 'pkg-list', 'pkgs-list',
+                'package', 'package-list', 'packages', 'packages-list')
         if extcmds and extcmds[0] not in cmds:
             base.logger.critical(_('Invalid history sub-command, use: %s.'),
                                  ", ".join(cmds))
@@ -1347,8 +1390,11 @@ class HistoryCommand(YumCommand):
             ret = base.historyInfoCmd(extcmds)
         elif vcmd == 'summary':
             ret = base.historySummaryCmd(extcmds)
-        elif vcmd == 'addon-info':
+        elif vcmd in ('addon', 'addon-info'):
             ret = base.historyAddonInfoCmd(extcmds)
+        elif vcmd in ('pkg', 'pkgs', 'pkg-list', 'pkgs-list',
+                      'package', 'package-list', 'packages', 'packages-list'):
+            ret = base.historyPackageListCmd(extcmds)
         elif vcmd == 'undo':
             ret = self._hcmd_undo(base, extcmds)
         elif vcmd in ('redo', 'repeat'):
@@ -1383,7 +1429,7 @@ class CheckRpmdbCommand(YumCommand):
             chkcmd = extcmds
 
         def _out(x):
-            print x
+            print to_unicode(x.__str__())
 
         rc = 0
         if base._rpmdb_warn_checks(out=_out, warn=False, chkcmd=chkcmd,
@@ -1394,3 +1440,31 @@ class CheckRpmdbCommand(YumCommand):
     def needTs(self, base, basecmd, extcmds):
         return False
 
+class LoadTransactionCommand(YumCommand):
+    def getNames(self):
+        return ['load-transaction', 'load-ts']
+
+    def getUsage(self):
+        return "filename"
+
+    def getSummary(self):
+        return _("load a saved transaction from filename")
+
+    def doCommand(self, base, basecmd, extcmds):
+        if not extcmds:
+            base.logger.critical(_("No saved transaction file specified."))
+            raise cli.CliError
+        
+        load_file = extcmds[0]
+        self.doneCommand(base, _("loading transaction from %s") % load_file)
+        
+        try:
+            base.load_ts(load_file)
+        except yum.Errors.YumBaseError, e:
+            return 1, [to_unicode(e)]
+        return 2, [_('Transaction loaded from %s with %s members') % (load_file, len(base.tsInfo.getMembers()))]
+
+
+    def needTs(self, base, basecmd, extcmds):
+        return True
+
